{=====================================================================}
{                                                                     }
{ Transparent PrintOut for Windows 95/NT ver 1.0.3 16/Jun./'97        }
{ http://www.kobira.co.jp/sakura/windows.htm                          }
{ author Seiji-Yasuda seiji@kobira.co.jp                              }
{                                                                     }
{ You may use it freely at your own risk in any kind of environment.  }
{                                                                     }
{ Note:                                                               }
{  ver 1.0.0 2/Dec./'96 Unit.                                         }
{  ver 1.0.1 8/Jan./'97 componet.                                     }
{  ver 1.0.2 11/Mar./'97 support Events.                              }
{  ver 1.0.3 16/Jun./'97 support DefaultMessage Property.             }
{                                                                     }
{=====================================================================}
unit tranprn;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DsgnIntf,WinSpool;

type
  TTranPrn = class(TComponent)
  private
    f_Filename       : TFileName;
    f_PrinterName    : String;
    f_DocumentName   : String;
    f_DefaultMessage : Boolean;

    f_OnBeforePrint          : TNotifyEvent;
    f_OnAfterPrint           : TNotifyEvent;
    f_OnAfterReadByte        : TNotifyEvent;

    f_OnOpenPrinterError     : TNotifyEvent;
    f_OnOpenFileError        : TNotifyEvent;
    f_OnStartDocPrinterError : TNotifyEvent;
    f_OnWritePrinterError    : TNotifyEvent;
    procedure ErrorEvent(f:TNotifyEvent;s:string);

  protected

  public
    constructor Create(AOwner:TComponent); override;
    destructor  destroy; override;
    function    Print:boolean;

  published
    // property.
    property FileName       : TFileName read f_Filename write f_Filename;
    property PrinterName    : String read f_PrinterName write f_PrinterName;
    property DocumentName   : String read f_DocumentName write f_DocumentName;
    property DefaultMessage : Boolean read f_DefaultMessage write f_DefaultMessage;

    // Event.
    property OnBeforePrint          : TNotifyEvent Read f_OnBeforePrint write f_OnBeforePrint;
    property OnAfterPrint           : TNotifyEvent Read f_OnAfterPrint   write f_OnAfterPrint;
    property OnAfterReadByte        : TNotifyEvent Read f_OnAfterReadByte write f_OnAfterReadByte;

    property OnOpenPrinterError     : TNotifyEvent Read f_OnOpenPrinterError write f_OnOpenPrinterError;
    property OnOpenFileError        : TNotifyEvent Read f_OnOpenFileError write f_OnOpenFileError;
    property OnStartDocPrinterError : TNotifyEvent Read f_OnStartDocPrinterError write f_OnStartDocPrinterError;
    property OnWritePrinterError    : TNotifyEvent Read f_OnWritePrinterError write f_OnWritePrinterError;
  end;

procedure Register;

implementation


//----------------------------------------------
// create.
constructor TTranPrn.Create(AOwner:TComponent);
begin
  inherited create(AOwner);
  f_DefaultMessage := True;
end;


//----------------------------------------------
// destroy.
destructor TTranPrn.Destroy;
begin
  inherited Destroy;
end;


//----------------------------------------------
// error event.
procedure TTranPrn.ErrorEvent(f:TNotifyEvent;s:string);
begin
  if Assigned(f) then
    f(self)
  else
    if f_DefaultMessage then
      ShowMessage(s);
end;


//----------------------------------------------
// execute print.
function TTranPrn.Print:boolean;
var
  pFileName:PChar; pDocName:PChar; pPrnName:PAnsiChar;
  hPrinter:THandle; DocInfo:TDocInfo1A; tmp:array[0..7] of Char; wcnt:DWORD;
  s:string; f:file; i:integer;
begin

  // event, before printe
  if Assigned(f_OnBeforePrint) then
    f_OnBeforePrint(self);

  // set value.
  pFileName := PChar(f_FileName);            // file name.
  pDocName  := PChar(f_DocumentName);        // document name.
  pPrnName  := PAnsiChar(f_PrinterName);     // printer name.

  // check document name.
  if pDocName^ = #0 then
    pDocName := pFileName;

  // get printer handle.
  if OpenPrinterA( pPrnName,hPrinter,nil ) = False then begin
     ErrorEvent(F_OnOpenPrinterError, 'Error:OpenPrinter!' );
     Result := False;
     exit;
  end;

  // init DocInfo.
  DocInfo.pDocName    := pDocName;   // document name.
  DocInfo.pOutputFile := NIL;        // output filename.
  DocInfo.pDatatype   := NIL;        // data type.

  // open file.
  try
    AssignFile( f, pFileName );
    Reset( f, 1 );
  except
    ErrorEvent(F_OnOpenFileError, 'Error:OpenFile!' );
    Result := False;
    exit;
  end;

  // start print spool. DocInfo structure 1 type. NT3.51 only type 1.
  if StartDocPrinterA( hPrinter,1,@DocInfo) = 0 then begin
     ErrorEvent( F_OnStartDocPrinterError, 'Error:StartDocPrinter!');
     Result := False;
     exit;
  end;

  // loop of reading file and spooling file.
  while not eof(f) do begin

    // process.
    application.ProcessMessages;

    // read byte.
    i := 1;
    BlockRead( f, tmp, i );

    // event, After printe
    if Assigned(f_OnAfterReadByte) then
      f_OnAfterReadByte(self);

    // set printer spooler
    if WritePrinter( hPrinter, @tmp, i, wcnt ) = False then begin
       ErrorEvent( F_OnWritePrinterError, 'Error:WritePrinter!');
       AbortPrinter( hPrinter );
       EndDocPrinter( hPrinter);
       ClosePrinter( hPrinter );
       Result := False;
       exit;
    end;

  end;

  // close file
  CloseFile(f);

  // end spool.
  // If the function fails, the return value should be FALSE.
  // To get extended error information, call GetLastError.
  Result := EndDocPrinter(hPrinter);
  if not Result then
    ErrorEvent( F_OnWritePrinterError, 'Error:EndDocPrinter!');

  // close printer handle.
  ClosePrinter( hPrinter );

  // event, After print
  if Result and Assigned(f_OnAfterPrint) then
    f_OnAfterPrint(self);
end;



//----------------------------------------------------------
// register
procedure Register;
begin
  RegisterComponents('Samples', [TTranPrn]);
end;



end.
